/*
 *  file      : src/ntb_gfxboot.c
 *  project   : b2im
 *  copyright : (C) 2015 by BULIN Claude
 * 
 *  This file is part of b2im project
 * 
 *  b2im is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; version 3.
 * 
 *  b2im is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 *  USA
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_b2im/lib.h"
#endif

#include "global.h"
#include "configuser.h"
#include "dragNdrop.h"





typedef struct {
	gint			width, height;
	gint			image_width, image_height;
	GtkWidget		*AdrDrawingarea;
	GdkPixbuf		*PixbufOriginal;
	cairo_surface_t	*image;
	gdouble			scale;
	gdouble			scale_w, scale_h;
	gdouble			X0;
	gdouble			Y0;
	GtkAllocation	allocation;
	GtkWidget		*button_mainmenu_normal_fg;
	GtkWidget		*button_mainmenu_selected_fg;
	GtkWidget		*button_mainmenu_bar_color;
	GList			*list_image_boot;
	GtkWidget		*adr_combo_image_boot;
} VAR_GFXBOOT;

VAR_GFXBOOT VarGfxboot;



// NOTEBOOK_GFXBOOT


void on_checkbutton_import_picture_to_gfxboot_toggled( GtkToggleButton *togglebutton, gpointer user_data )
{
	gboolean	bool_activate = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(togglebutton) );
	
	if( TRUE == bool_activate ) {
		if( FALSE == g_file_test( "/usr/bin/gfxboot", G_FILE_TEST_IS_REGULAR )) {
			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), FALSE );
			wind_info_init (
				WindMain_wind_main,
				_(" Warning "),
				_("The gxfboot package must be installed."),
				  "     ",
				  "\n",
				"");
			bool_activate = FALSE;
		}
	}
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("box_import_picture_to_gfxboot")), bool_activate );
}

void on_checkbutton_import_picture_to_gfxboot_realize( GtkWidget *widget, gpointer user_data )
{
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(widget), Config.bool_import_picture );
	on_checkbutton_import_picture_to_gfxboot_toggled( GTK_TOGGLE_BUTTON(widget), NULL );
			
}

gboolean on_drawingarea_img_boot_draw( GtkWidget *widget, cairo_t *cr, gpointer user_data )
{
	if( NULL != VarGfxboot.AdrDrawingarea ) {
		
		gdouble	size_font;
		gint	pos_x, pos_y;
		
		VarGfxboot.image_width  = gdk_pixbuf_get_width( VarGfxboot.PixbufOriginal );
		VarGfxboot.image_height = gdk_pixbuf_get_height( VarGfxboot.PixbufOriginal );
		
		gtk_widget_get_allocation( VarGfxboot.AdrDrawingarea, &VarGfxboot.allocation );
		VarGfxboot.scale_w = (gdouble)VarGfxboot.allocation.width / (gdouble)VarGfxboot.image_width;
		
		VarGfxboot.scale_h = (gdouble)VarGfxboot.allocation.height / (gdouble)VarGfxboot.image_height;
		VarGfxboot.scale = MIN(VarGfxboot.scale_w, VarGfxboot.scale_h);
	
		VarGfxboot.X0 = (((gdouble)VarGfxboot.allocation.width / VarGfxboot.scale) - (gdouble)VarGfxboot.image_width) / 2.0;
		VarGfxboot.Y0 = (((gdouble)VarGfxboot.allocation.height / VarGfxboot.scale) - (gdouble)VarGfxboot.image_height) / 2.0;

		// cairo_save( cr );
		cairo_scale( cr, VarGfxboot.scale, VarGfxboot.scale );
		cairo_translate( cr, VarGfxboot.X0, VarGfxboot.Y0 );
		gdk_cairo_set_source_pixbuf( cr, VarGfxboot.PixbufOriginal, 0.0, 0.0 );
		cairo_paint( cr );
		// cairo_restore( cr );
	
		cairo_save( cr );
		cairo_select_font_face( cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD );
		size_font = 14;
		cairo_set_font_size( cr, size_font );
		
		pos_x = 180;
		pos_y = size_font + ( VarGfxboot.image_height / 4 ) + 65;
		cairo_set_source_rgba( cr,
					Config.mainmenu_bar_color_list_0,
					Config.mainmenu_bar_color_list_1,
					Config.mainmenu_bar_color_list_2,
					1.0
					);
		cairo_rectangle( cr,
				pos_x - 10,
				pos_y - size_font - 5,
				350,
				size_font + 15
				);			
		cairo_fill( cr );
	
		cairo_set_source_rgb( cr,
				Config.mainmenu_selected_fg_list_0,
				Config.mainmenu_selected_fg_list_1,
				Config.mainmenu_selected_fg_list_2
				);
		cairo_move_to( cr, pos_x, pos_y );
		cairo_show_text( cr, _("Start Manjaro Linux"));					// Démarrer Manjaro Linux
		
		cairo_set_source_rgb( cr,
				Config.mainmenu_normal_fg_list_0,
				Config.mainmenu_normal_fg_list_1,
				Config.mainmenu_normal_fg_list_2
				);
		pos_y += size_font + 10;
		cairo_move_to( cr, pos_x, pos_y );
		cairo_show_text( cr, _("Start (non-free drivers)"));			// Démarrer (avec des pilotes propriétaires)
		pos_y += size_font + 10;
		cairo_move_to( cr, pos_x, pos_y );
		cairo_show_text( cr, _("Boot from Hard Disk"));					// Amorcer depuis le disque dur
		pos_y += size_font + 10;
		cairo_move_to( cr, pos_x, pos_y );
		cairo_show_text( cr, _("Hardware Detection Tool"));				// Outil de détection du matériel
		pos_y += size_font + 10;
		cairo_move_to( cr, pos_x, pos_y );
		cairo_show_text( cr, _("Memory Test"));							// Test de la mémoire
		cairo_restore( cr );
	}
	return( FALSE );
}

// 
// 
void NtbGfxBoot_add_img_file_to_Drawingarea( GSList *p_list )
{
	GSList		*gs_List = p_list;
	gchar		*Ptr = NULL;

	if( FALSE == g_file_test( "/usr/bin/gfxboot", G_FILE_TEST_IS_REGULAR )) {
		wind_info_init (
			WindMain_wind_main,
			_(" Warning "),
			_("The gxfboot package must be installed."),
			  "     ",
			  "\n",
			"");
		return;
	}
	while( gs_List ) {
		if( (Ptr = (gchar *)gs_List->data) != NULL ) {
			// PRENDRE UNIQUEMENT LA PREMIERE IMAGE VALIDE DANS LA LISTE
			if( TRUE == FileIs_image( Ptr )) {
				
				VarGfxboot.list_image_boot = g_list_append( VarGfxboot.list_image_boot, g_strdup_printf("%s", Ptr ));
				gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(VarGfxboot.adr_combo_image_boot), g_path_get_basename( Ptr ) );
				
				// VarGfxboot.PixbufOriginal = gdk_pixbuf_new_from_file_at_scale( Ptr, 640, 480, TRUE, NULL );
				// gtk_widget_queue_draw( VarGfxboot.AdrDrawingarea );	
				
				gtk_combo_box_set_active( GTK_COMBO_BOX (VarGfxboot.adr_combo_image_boot), g_list_length( VarGfxboot.list_image_boot ) -1 );
				return;
			}
		}
		gs_List = g_slist_next( gs_List );
	}
}
// 
// 
static void NtbGfxBoot_drag_data_received(
					GtkWidget        *widget,
					GdkDragContext   *context,
					gint              x,
					gint              y,
					GtkSelectionData *data,
					guint             info,
					guint             time,
					gpointer          user_data)
{
	// PRINT("DND FROM DESKTOP TO DRAWING"); 
	// Une copie ne peut aller vers elle meme !!!
	if (gtk_drag_get_source_widget(context) != widget) {
		// dragndrop_list_drag_data (widget, (gchar *)data->data);
		dragndrop_list_drag_data( widget, (gchar*)gtk_selection_data_get_data( data ));
	}
}
// 
// 
void on_drawingarea_img_boot_realize( GtkWidget *widget, gpointer user_data )
{
	gchar		*PathName = xdg_search_data_xdg( "welcome.jpg" );
	
	VarGfxboot.AdrDrawingarea = widget;
	VarGfxboot.width  = gtk_widget_get_allocated_width( widget );
	VarGfxboot.height = gtk_widget_get_allocated_height( widget );
	VarGfxboot.image = cairo_image_surface_create(
				CAIRO_FORMAT_ARGB32,
				VarGfxboot.width,
				VarGfxboot.height
				);
	// VarGfxboot.PixbufOriginal = gdk_pixbuf_new_from_file_at_scale( "/home/cat/work/img_iso/welcome.jpg", 640, 480, TRUE, NULL );
	// VarGfxboot.PixbufOriginal = gdk_pixbuf_new_from_file_at_scale( "/home/cat/work/img_iso/m-xfce.jpg", 640, 480, TRUE, NULL );
	VarGfxboot.PixbufOriginal = gdk_pixbuf_new_from_file_at_scale( PathName, 640, 480, TRUE, NULL );
	gtk_widget_queue_draw( VarGfxboot.AdrDrawingarea );
	g_free( PathName );
	
	// Drag and drop support
	// SIGNAL : 'drag-data-received'
	gtk_drag_dest_set (GTK_WIDGET (widget),
			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP | GTK_DEST_DEFAULT_HIGHLIGHT,
			   drag_types,
			   n_drag_types,
			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_DEFAULT
			   );
	g_signal_connect(G_OBJECT(widget), "drag-data-received", G_CALLBACK(NtbGfxBoot_drag_data_received), NULL);
}

void on_combobox_choice_img_desktop_changed( GtkComboBox *combobox, gpointer user_data )
{
	gchar	*text_combo = NULL;
	gchar	*pathname = NULL;
	gchar	*ptr_data_list = NULL;
	
	if( NULL == VarGfxboot.adr_combo_image_boot ) return;
	
	Config.choice_num_image_boot = gtk_combo_box_get_active( GTK_COMBO_BOX (combobox) );
	
	text_combo = gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT (combobox) );
	
	ptr_data_list = g_list_nth_data( VarGfxboot.list_image_boot, Config.choice_num_image_boot );
	pathname = g_strdup_printf("%s", (gchar *)ptr_data_list );
	
	g_object_unref( VarGfxboot.PixbufOriginal );
	VarGfxboot.PixbufOriginal = gdk_pixbuf_new_from_file_at_scale( pathname, 640, 480, TRUE, NULL );
	gtk_widget_queue_draw( VarGfxboot.AdrDrawingarea );	
	
	g_free( text_combo );
	g_free( pathname );
}

gint NtbGfxBoot_compar( gchar * a, gchar * b )
{
	return strcmp( (char*)a, (char*)b );
}


void on_combobox_choice_img_desktop_realize( GtkWidget *widget, gpointer user_data )
{
	DIR			*dp;
	struct dirent *entry;
	struct stat statbuf;
	gchar		*PathName = xdg_search_data_xdg( "welcome.jpg" );
	gchar		*dirname_image_boot = g_path_get_dirname( PathName );
	GList		*list = NULL;
	gchar		*Ptr = NULL;
	gchar		*Welcome = NULL;
	
	g_free( PathName );
	if( NULL == (dp = opendir( dirname_image_boot )) ) {
		// g_free( DirName );
		if( NULL != dirname_image_boot )
			g_free( dirname_image_boot );
		return;
	}
	chdir( dirname_image_boot );
	while( (entry = readdir(dp)) != NULL ) {
		lstat( entry->d_name, &statbuf );
		if( S_ISDIR( statbuf.st_mode )) {
			if( strcmp( ".", entry->d_name ) == 0 || strcmp( "..", entry->d_name ) == 0 ) continue;
		}
		else {
			if( strcmp( "welcome.jpg", entry->d_name ) == 0 ) {
				Welcome = g_strdup_printf("%s/%s", dirname_image_boot, entry->d_name );
			}
			else {
				VarGfxboot.list_image_boot = g_list_append( VarGfxboot.list_image_boot, g_strdup_printf("%s/%s", dirname_image_boot, entry->d_name ));
			}
		}
	}
	closedir( dp );
	
	// Sort List
	VarGfxboot.list_image_boot = g_list_sort( VarGfxboot.list_image_boot, (GCompareFunc)NtbGfxBoot_compar );
	VarGfxboot.list_image_boot = g_list_prepend( VarGfxboot.list_image_boot, g_strdup( Welcome ) );
	g_free( Welcome );
	Welcome = NULL;
	
	if( NULL != ( list = g_list_first( VarGfxboot.list_image_boot ))) {
		while( list ) {
			if( NULL != list->data ) {
				Ptr = strrchr( list->data, '/' );
				Ptr ++;
				gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(widget), Ptr );
			}
			list = g_list_next( list );
		}
	}
	
	// g_free( DirName );
	VarGfxboot.adr_combo_image_boot = widget;
	gtk_combo_box_set_active( GTK_COMBO_BOX (widget), Config.choice_num_image_boot );
	g_free( dirname_image_boot );
}

void on_combobox_gfxboot_langue_realize( GtkWidget *widget, gpointer user_data )
{
	gchar *countries[] = { "af_ZA", "ar_EG", "ast_ES", "bn_BD", "bs_BA", "bg_BG", "ca_ES", "cs_CZ", "cy_GB", "da_DK",
					"de_BE", "de_DE", "et_EE", "en_GB", "en_US", "es_ES", "fa_IR", "fr_BE", "fr_FR", "gl_ES",
					"ka_GE", "gu_IN", "el_GR", "hi_IN", "id_ID", "hr_HR", "it_IT", "he_IL", "ja_JP", "jv_ID",
					"km_KH", "ko_KR", "ky_KG", "lo_LA", "lt_LT", "mr_IN", "hu_HU", "mk_MK", "nl_BE", "nl_NL",
					"nb_NO", "nn_NO", "pl_PL", "pt_PT", "pt_BR", "pa_IN", "ro_RO", "ru_RU", "zh_CN", "si_LK",
					"sk_SK", "sl_SI", "sr_RS", "fi_FI", "sv_SE", "tg_TJ", "ta_IN", "th_TH", "vi_VN", "zh_TW",
					"tr_TR", "uk_UA", "wa_BE", "xh_ZA", "zu_ZA", "" };
	gint Cpt;
	
	for( Cpt = 0; *countries[ Cpt ] != '\0'; Cpt ++ ) 
		gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(widget), countries[ Cpt ] );
	gtk_combo_box_set_active( GTK_COMBO_BOX (widget), Config.choice_num_language );
}

gboolean on_drawingarea_button_mainmenu_fg_draw( GtkWidget *widget, cairo_t *cr, gpointer user_data )
{
	cairo_save( cr );
	// contour / around
	cairo_set_source_rgba( cr, 0., 0., 0., 1.0 );
	gtk_widget_get_allocation( widget, &VarGfxboot.allocation );
	cairo_rectangle( cr, 0, 0, VarGfxboot.allocation.width, VarGfxboot.allocation.height );			
	cairo_fill( cr );
	// intérieur / inside
	if( widget == VarGfxboot.button_mainmenu_normal_fg ) {
		cairo_set_source_rgba(
				cr,
				Config.mainmenu_normal_fg_list_0,
				Config.mainmenu_normal_fg_list_1,
				Config.mainmenu_normal_fg_list_2,
				1.0
				);
	}
	else if( widget == VarGfxboot.button_mainmenu_selected_fg ) {
		cairo_set_source_rgba(
				cr,
				Config.mainmenu_selected_fg_list_0,
				Config.mainmenu_selected_fg_list_1,
				Config.mainmenu_selected_fg_list_2,
				1.0
				);
	}
	else if( widget == VarGfxboot.button_mainmenu_bar_color ) {
		cairo_set_source_rgba(
				cr,
				Config.mainmenu_bar_color_list_0,
				Config.mainmenu_bar_color_list_1,
				Config.mainmenu_bar_color_list_2,
				1.0
				);
	}
	cairo_rectangle( cr, 1, 1, VarGfxboot.allocation.width-2, VarGfxboot.allocation.height-2 );			
	cairo_fill( cr );
	cairo_restore( cr );
	return( FALSE );
}

void on_button_mainmenu_color_clicked( GtkButton *button, gpointer user_data )
{
	GtkWidget		*dialog;
	GtkResponseType result;
	GdkRGBA			color;

	dialog = gtk_color_chooser_dialog_new ("Changing color", GTK_WINDOW (WindMain_wind_main));
	// gtk_window_set_transient_for (GTK_WINDOW(widget), GTK_WINDOW(WindMain_wind_main));
	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE );
	result = gtk_dialog_run(GTK_DIALOG(dialog));
	if (result == GTK_RESPONSE_OK) {
		gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog), &color);
		if( button == GTK_BUTTON(GLADE_GET_OBJECT("button_mainmenu_normal_fg")) ) {
			Config.mainmenu_normal_fg_list_0 = color.red;
			Config.mainmenu_normal_fg_list_1 = color.green;
			Config.mainmenu_normal_fg_list_2 = color.blue;
			gtk_widget_queue_draw( VarGfxboot.button_mainmenu_normal_fg );
		}
		else if( button == GTK_BUTTON(GLADE_GET_OBJECT("button_mainmenu_selected_fg")) ) {
			// gtk_widget_override_background_color( VarGfxboot.button_mainmenu_selected_fg, 0, &color);
			Config.mainmenu_selected_fg_list_0 = color.red;
			Config.mainmenu_selected_fg_list_1 = color.green;
			Config.mainmenu_selected_fg_list_2 = color.blue;
			gtk_widget_queue_draw( VarGfxboot.button_mainmenu_selected_fg );
		}
		else if( button == GTK_BUTTON(GLADE_GET_OBJECT("button_mainmenu_bar_color")) ) {
			// gtk_widget_override_background_color(VarGfxboot.button_mainmenu_bar_color, 0, &color);
			Config.mainmenu_bar_color_list_0 = color.red;
			Config.mainmenu_bar_color_list_1 = color.green;
			Config.mainmenu_bar_color_list_2 = color.blue;
			gtk_widget_queue_draw( VarGfxboot.button_mainmenu_bar_color );
		}
	}
	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (dialog), &color);
	gtk_widget_destroy (GTK_WIDGET (dialog));
}

void on_drawingarea_button_mainmenu_color_realize( GtkWidget *widget, gpointer user_data )
{
	if( widget == GTK_WIDGET(GLADE_GET_OBJECT("drawingarea_button_mainmenu_normal_fg")) ) {
		VarGfxboot.button_mainmenu_normal_fg = widget;
	}
	else if( widget == GTK_WIDGET(GLADE_GET_OBJECT("drawingarea_button_mainmenu_selected_fg")) ) {
		VarGfxboot.button_mainmenu_selected_fg = widget;
	}
	else if( widget == GTK_WIDGET(GLADE_GET_OBJECT("drawingarea_button_mainmenu_bar_color")) ) {
		VarGfxboot.button_mainmenu_bar_color = widget;
	}
	gtk_widget_queue_draw( widget );
}

void NtbGfxBoot_get_from_server_GFXBOOT( void )
{
	gchar	*dirname = g_get_current_dir();
	gchar	*LineCommand = NULL;
	
	printf("%s*-*-> NtbGfxBoot_get_from_server_GFXBOOT%s\n", BOLD_YELLOW, RESET );
	
	if( TRUE == libutils_remove_rep( "/tmp/gfxboot" ))
		printf("%s      Remove [ /tmp/gfxboot ] %s\n", BOLD_YELLOW, RESET );
	chdir( "/tmp" );
	LineCommand = g_strdup_printf( "git clone https://github.com/manjaro/artwork-gfxboot" );
	system (LineCommand);
	g_free (LineCommand);
	LineCommand = NULL;
	chdir( dirname );
	g_rename( "/tmp/artwork-gfxboot", "/tmp/gfxboot" );
}

void NtbGfxBoot_convert_picture( gchar *_size_picture, gchar *_src_file, gchar *_dest_file, size_t _size_request_maxi )
{
	gint	_quality = 24;
	gchar	*LineCommand = NULL;
	
	while( TRUE ) {
		LineCommand = g_strdup_printf( "convert -resize %s -quality %d %s %s", _size_picture, _quality, _src_file, _dest_file );
		printf("      %s\n", LineCommand );
		system( LineCommand );
		g_free( LineCommand );
		LineCommand = NULL;
		if( _size_request_maxi > libutils_get_size_file( _dest_file )) break;
		_quality --;
		if( _quality <= 0 ) break;
	}
}

void NtbGfxBoot_gfxboot_convert_pictures( void )
{
	printf("%s*-*-> NtbGfxBoot_gfxboot_make_pictures%s\n", BOLD_YELLOW, RESET );
	
	if( TRUE == g_file_test( "/usr/bin/convert", G_FILE_TEST_EXISTS )) {
		
		gchar	*src_file = NULL;
		
		Config.choice_num_image_boot = gtk_combo_box_get_active( GTK_COMBO_BOX (VarGfxboot.adr_combo_image_boot) );
		src_file = g_list_nth_data( VarGfxboot.list_image_boot, Config.choice_num_image_boot );
		
		NtbGfxBoot_convert_picture( "800x600!", src_file, "/tmp/gfxboot/data/welcome.jpg", 16384 );
		NtbGfxBoot_convert_picture( "800x600!", src_file, "/tmp/gfxboot/data/back800x600.jpg", 16384 );
		NtbGfxBoot_convert_picture( "640x480!", src_file, "/tmp/gfxboot/data/back640x480.jpg", 11264 );
		
	}
	else {
		// PUT ERROR
		printf("%s*-*-> [ convert ] not found !%s\n", BOLD_RED, RESET );
	}
}

void NtbGfxBoot_gfxboot_kolourpaint_pictures( void )
{
	printf("%s*-*-> NtbGfxBoot_gfxboot_kolourpaint_pictures%s\n", BOLD_YELLOW, RESET );
	
	if( TRUE == g_file_test( "/usr/bin/kolourpaint", G_FILE_TEST_EXISTS )) {
				
		system( "/usr/bin/kolourpaint /tmp/gfxboot/data/welcome.jpg /tmp/gfxboot/data/back800x600.jpg /tmp/gfxboot/data/back640x480.jpg &>/dev/null" );
	}
	else {
		// PUT ERROR
		printf("%s*-*-> [ kolourpaint ] not found !%s\n", BOLD_RED, RESET );
	}
}

void NtbGfxBoot_gfxboot_make( void )
{
	FILE 	*fp = NULL;
	gint 	r,g,b;
	gchar	*home_isolinux = g_strdup_printf("%s/shared/isolinux", Config.path_folder_manjaro_profiles );
	gchar	*home_save_isolinux = g_strdup_printf("%s/shared/save_isolinux", Config.path_folder_manjaro_profiles );
	
	printf("%s*-*-> NtbGfxBoot_gfxboot_make%s\n", BOLD_YELLOW, RESET );
	
	fp = fopen( "/tmp/Processing_files.sh", "w" );
	fprintf( fp, "#!/bin/bash\n" );
	fprintf( fp, "#\n\n" );
	fprintf( fp, "cd /tmp/gfxboot/src\n" );
	// color menu lines
	r = Config.mainmenu_normal_fg_list_0 * 255.0;
	g = Config.mainmenu_normal_fg_list_1 * 255.0;
	b = Config.mainmenu_normal_fg_list_2 * 255.0;
	fprintf( fp, "sed -i 's/mainmenu.normal.fg=.*/mainmenu.normal.fg=0x%02X%02X%02X/' gfxboot.cfg\n", r, g, b );
	r = Config.mainmenu_selected_fg_list_0 * 255.0;
	g = Config.mainmenu_selected_fg_list_1 * 255.0;
	b = Config.mainmenu_selected_fg_list_2 * 255.0;
	fprintf( fp, "sed -i 's/mainmenu.selected.fg=.*/mainmenu.selected.fg=0x%02X%02X%02X/' gfxboot.cfg\n", r, g, b );
	r = Config.mainmenu_bar_color_list_0 * 255.0;
	g = Config.mainmenu_bar_color_list_1 * 255.0;
	b = Config.mainmenu_bar_color_list_2 * 255.0;
	fprintf( fp, "sed -i 's/mainmenu.bar.color=.*/mainmenu.bar.color==0x%02X%02X%02X/' gfxboot.cfg\n", r, g, b );
	fprintf( fp, "cd /tmp/gfxboot\n" );
	// language choice
	fprintf( fp, "sed -i 's/DEFAULT_LANG =.*/DEFAULT_LANG = %s/' ./Makefile\n",
		gtk_combo_box_text_get_active_text( GTK_COMBO_BOX_TEXT(GLADE_GET_OBJECT("combobox_gfxboot_langue"))));
	fprintf( fp, "make\n" );
	fprintf( fp, "cp -a %s %s\n", home_isolinux, home_save_isolinux );
	fprintf( fp, "rm -rf %s\n", home_isolinux );
	fprintf( fp, "cp -a /tmp/gfxboot/isolinux %s\n", home_isolinux );
	fprintf( fp, "\n" );
	fclose( fp );
	g_free( home_isolinux );
	g_free( home_save_isolinux );
	g_chmod( "/tmp/Processing_files.sh", 0700 );   
	
	system( "/tmp/Processing_files.sh" );
	
	g_unlink( "/tmp/Processing_files.sh" );
	libutils_remove_rep( "/tmp/gfxboot" );
	libutils_remove_rep( "/tmp/artwork-gfxboot" );
}

void NtbGfxBoot_gfxboot_action( void )
{
	if( TRUE == gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(GLADE_GET_OBJECT("checkbutton_import_picture_to_gfxboot")) )) {
		printf("%s*-*-> NtbGfxBoot_gfxboot_action%s\n", BOLD_YELLOW, RESET );
		NtbGfxBoot_get_from_server_GFXBOOT();
		NtbGfxBoot_gfxboot_convert_pictures();
		NtbGfxBoot_gfxboot_kolourpaint_pictures();
		NtbGfxBoot_gfxboot_make();
	}
}
